/*
 * file SimpleEdit.java
 * 
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.SimpleEdit
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
*/

package com.ibm.rational.stp.client.samples;

import static com.ibm.rational.stp.client.samples.SimpleQuery.g_callback;
import static com.ibm.rational.wvcm.stp.cq.CqFieldValue.ValidationStatus.INVALID;
import static com.ibm.rational.wvcm.stp.cq.CqProvider.CQ_ONLY_PROVIDER_CLASS;
import static com.ibm.rational.wvcm.stp.cq.CqProvider.DELIVER;
import static com.ibm.rational.wvcm.stp.cq.CqProvider.HOLD;
import static javax.swing.JOptionPane.INFORMATION_MESSAGE;
import static javax.swing.JOptionPane.OK_CANCEL_OPTION;
import static javax.swing.JOptionPane.OK_OPTION;
import static javax.swing.JOptionPane.showConfirmDialog;
import static javax.swing.JOptionPane.showInputDialog;
import static javax.swing.JOptionPane.showMessageDialog;

import java.util.Arrays;
import java.util.List;

import javax.wvcm.Location;
import javax.wvcm.ProviderFactory;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyRequestItem.PropertyRequest;

import com.ibm.rational.wvcm.stp.StpLocation.Namespace;
import com.ibm.rational.wvcm.stp.cq.CqAction;
import com.ibm.rational.wvcm.stp.cq.CqContextResource;
import com.ibm.rational.wvcm.stp.cq.CqFieldDefinition;
import com.ibm.rational.wvcm.stp.cq.CqFieldValue;
import com.ibm.rational.wvcm.stp.cq.CqProvider;
import com.ibm.rational.wvcm.stp.cq.CqRecord;

/**
 * Sample CM API Application that allows a ClearQuest record to be edited
 * interactively.
 */
public class SimpleEdit
{
    private static final int ERROR = javax.swing.JOptionPane.ERROR_MESSAGE;
    private static final String SUBTITLE = 
        "\nEnter field=value;..., or leave blank to finish";
    
/**
 * @param args arg[0] == simple name of edit action; arg[1]=record to edit
 */
public static void main(String[] args) throws Exception {
    while (args.length != 2) {
        String cmd = showInputDialog("Enter <action> <record>",
                                     Arrays.deepToString(args));
        
        if (cmd == null)
            return;
        
        args = cmd.split("\\s+");
        
        if (args.length == 1)
            showRecord(args[0]);
    }
    
    edit(args);
}

public static void edit(String[] args) throws Exception {
    CqProvider provider = (CqProvider) ProviderFactory
                           .createProvider(CQ_ONLY_PROVIDER_CLASS, g_callback);
    CqRecord record = provider.buildProxy(CqRecord.class, args[1]);
    String fields, title = args[0] + " " + record;
    Location actionLoc = record.stpLocation()
              .recomposeWithNamespace(Namespace.ACTION).parent().child(args[0]);

    record.setAction(provider.buildProxy(CqAction.class, actionLoc));    
    
    while (null != (fields = showInputDialog(null, status(record) + SUBTITLE,
                                             title, OK_CANCEL_OPTION))) 
        try {
            if ((fields = fields.trim()).length() > 0) {
                for (String set : fields.split(";", -2)) {
                    String[] nv = set.split("=");
                    record.setField(nv[0].trim(), nv.length > 1? nv[1]: null);
                }
                record = (CqRecord) record.doWriteProperties(ALL_FIELDS, HOLD);
            } else {
                record = (CqRecord) record.doDeliver(ALL_FIELDS);
                showMessageDialog(null, "Success!\n" + status(record),
                                  title, INFORMATION_MESSAGE);
                break;
            }
        } catch (WvcmException e) {
            if (OK_OPTION != showConfirmDialog(null, e + "\nContinue?",
                                               title, OK_CANCEL_OPTION, ERROR))
                break;
            record = provider.cqRecord(record.stpLocation());
        }
    
    provider.terminate();
    System.exit(0);
}

    static String status(CqRecord record) 
    throws WvcmException
    {
        StringBuffer buf = new StringBuffer();
        
        if (!record.hasProperties(ALL_FIELDS))
            record = (CqRecord)record.doWriteProperties(ALL_FIELDS, HOLD);

        for (CqFieldValue field: record.getAllFieldValues())
            if (!field.getFieldDefinition().getIsSystemOwned())
                buf.append((!field.getValueChangedThisAction()? "=== [": 
                            field.getValidationStatus()==INVALID? "!!!!!!! [": "+++ [") 
                            + field.getRequiredness().name().charAt(0) + "] "
                            + field.getName() + " = " + field.getValue() + "\n");
        
        return buf.toString();
    }
    
    private static final PropertyRequest ALL_FIELDS = 
            new PropertyRequest(CqRecord.ALL_FIELD_VALUES
                                    .nest(CqFieldValue.VALUE
                                          .nest(CqFieldValue.VALUE, 
                                                CqFieldValue.VALUE_CHANGED_THIS_ACTION,
                                                CqFieldValue.VALIDATION_STATUS,
                                                CqFieldValue.REQUIREDNESS,
                                                CqFieldValue.FIELD_DEFINITION
                                                    .nest(CqFieldDefinition
                                                          .IS_SYSTEM_OWNED))));

    // Similar to edit method, but allows a trailing ';' to signal end of update
    // so last batch of record updates are written with delivery
    public static void edit2(String[] args) throws Exception {
    CqProvider provider = (CqProvider) ProviderFactory
                           .createProvider(CQ_ONLY_PROVIDER_CLASS, g_callback);
    CqRecord record = provider.buildProxy(CqRecord.class, args[1]);
    String fields, title = args[0] + " " + record;
    Location actionLoc = record.stpLocation()
              .recomposeWithNamespace(Namespace.ACTION).parent().child(args[0]);

    record.setAction(provider.buildProxy(CqAction.class, actionLoc));    
    
    while (null != (fields = showInputDialog(null, status(record) + SUBTITLE,
                                             title, OK_CANCEL_OPTION))) 
        try {
            if (!(fields = fields.trim()).equals(""))
                for (String set : fields.split("\\s*;\\s*")) {
                    String[] nv = set.split("\\s*=\\s*");
                    record.setField(nv[0], nv.length > 1? nv[1]: null);
                }
            
            if (!fields.matches("|.*;")) {
                record = (CqRecord) record.doWriteProperties(ALL_FIELDS, HOLD);    
            } else {
                record = (CqRecord) record.doWriteProperties(ALL_FIELDS, DELIVER);
                showMessageDialog(null, "Success!\n" + status(record),
                                  title, INFORMATION_MESSAGE);
                break;
            }
        } catch (WvcmException e) {
            if (OK_OPTION != showConfirmDialog(null, e + "\nContinue?",
                                               title, OK_CANCEL_OPTION, ERROR))
                break;
            record = provider.cqRecord(record.stpLocation());
        }
    
    provider.terminate();
    System.exit(0);
}
    
static void showRecord(String location) throws Exception {
    CqProvider provider = (CqProvider) ProviderFactory
                           .createProvider(CQ_ONLY_PROVIDER_CLASS, g_callback);
    CqRecord record = provider.buildProxy(CqRecord.class, location);
    
    PropertyRequest wantedProperties = new PropertyRequest
        (CqRecord.LEGAL_ACTIONS.nest(CqAction.DISPLAY_NAME),
         CqRecord.ALL_FIELD_VALUES
             .nest(CqFieldValue.VALUE
                   .nest(CqFieldValue.VALUE,
                         CqFieldValue.FIELD_DEFINITION
                             .nest(CqFieldDefinition.IS_SYSTEM_OWNED))));
    record = (CqRecord)record.doReadProperties(wantedProperties);

    StringBuffer buf = new StringBuffer("Fields...\n");
    
    for (CqFieldValue field: record.getAllFieldValues())
        if (!field.getFieldDefinition().getIsSystemOwned() || field.getName().equals("State"))
            buf.append("  " + field.getName() + " = " + field.getValue() + "\n");
    
    buf.append("\nLegal Actions...\n  ");
    
    for (CqAction action: record.getLegalActions())
        buf.append(action.getDisplayName() + ", ");
    
    showMessageDialog(null, buf.toString(), location, INFORMATION_MESSAGE); 
    provider.terminate();
}
}
